﻿Type=Class
Version=6
ModulesStructureVersion=1
B4A=true
@EndOfDesignText@
'Class module LimitBar

'Events declaration
#Event: ValuesChanged
#ExcludeFromLibrary: FALSE

'Class module
Sub Class_Globals
	Private fParent  As Object
	Private Callback As Object			' calling module
	Private ltbPanelBack As Panel		' the background panel BASE
	Private ltbPanelFront As Panel	' the foreground panel
	Private cvsPanelBack As Canvas	' the background canvas
	Private cvsPanelFront As Canvas	' the foreground canvas
	Private rectPanelFront As Rect	' a rectangle for the foreground canvas
	
	Private BackLineColor As Int		' color for the background line
	BackLineColor = Colors.Black
	Private FrontLineColor As Int		' color for the foreground line
	FrontLineColor = Colors.RGB(51, 181, 229)
	Private LimitSliderColor As Int	' color for the cursors
	LimitSliderColor = Colors.ARGB(180, 51, 181, 229)
	Private Margin = 15dip As Float	' left and right margins for the line
	Private x0, y0, x1 As Int				' values used internaly
	Private MaxVal = 100 As Int			' value of the Max property
	Private Scale As Double					' scale between position value and pixels
	Private cLeft, cTop, cWidth, cHeight As Int	' holds these properties
	Private cLimitLeft As Int				' value of the left limit
	Private cLimitRight As Int			' value of the right limit
	Private PositionPixels(2) As Int	' left and right positions in pixels
																		' 0 = left   1 = right
	Private PosIndex As Int					' current index of the position
	Private EventName As String			' event name
	Private Paths(2) As Path				' two paths for the cursor shape
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize(CallbackModule As Object, cEventName As String)
	Callback  = CallbackModule
	EventName = cEventName
End Sub

'
''Note: This sub never gets called +++++++
public Sub getParent(v As View) As View
  Private jo As JavaObject
	If v.IsInitialized Then
    Return jo.RunMethod("getParent", Null)
	Else
		Return fParent	
	End If
End Sub	


'Adds the LimitBar to the Parent object
'Parent = parent view, the Activity or a Panel
'cLeft, cRight, cWidth, cRight = position and dimensions properties of the LimitBar
'cColor = background color of the LimitBar
'cRadius = corner radius of the LimitBar
Public Sub AddToParent(Parent As Activity, Left As Int, Top As Int, Width As Int, Height As Int, cColor As Int, cRadius As Int)
	cLeft = Left
	cTop = Top
	cWidth = Width
	cHeight = Height
	fParent = Parent
	
	cHeight = Max(cHeight, 30dip)				' limits the height to min 30dip 
	cRadius = Min(cRadius, cHeight / 2)	' limits the max radius to half the height
	
	' initialize the background panel ltbPanelBack and add it onto the parent view
	ltbPanelBack.Initialize("")
	Parent.AddView(ltbPanelBack, cLeft, cTop, cWidth, cHeight)
	' initialize and set the ColorDrawable for the background panel
	Dim cdw As ColorDrawable
	cdw.Initialize(cColor, cRadius)
	ltbPanelBack.Background = cdw
	
	' initialize the foreground panel ltbPanelFront and add it onto the parent view
	ltbPanelFront.Initialize("ltbPanelFront")
	Parent.AddView(ltbPanelFront, cLeft, cTop, cWidth, cHeight)
	' initialize the foreground panel rectangle used to erase ltbPanelFront
	rectPanelFront.Initialize(0, 0, ltbPanelFront.Width, ltbPanelFront.Height)
	ltbPanelFront.BringToFront	
	
	Init
End Sub

	
public Sub GetBase As Panel
	Return ltbPanelBack	
End Sub
	

Public Sub DesignerCreateView(Base As Panel, Lbl As Label, Props As Map)
	' we memorize the properties

	Private r As Reflector
	r.target = Base
	fParent  = r.RunMethod("getParent")
	
	cLeft    = Base.Left
	cTop     = Base.Top
	cWidth   = Base.Width
	cHeight  = Base.Height
		
	' we use the Base panel as the background panel
	ltbPanelBack = Base
	
	' initialize the foreground panel and add it onto the background panel
	ltbPanelFront.Initialize("ltbPanelFront")
	ltbPanelBack.AddView(ltbPanelFront, 0, 0, ltbPanelBack.Width, ltbPanelBack.Height)
	' initialize the foreground panel rectangle used to erase ltbPanelFront
	rectPanelFront.Initialize(0, 0, ltbPanelFront.Width, ltbPanelFront.Height)
	ltbPanelFront.BringToFront	
	
	Init
End Sub


Private Sub Init
	' set local variables 
	x0 = Margin
	x1 = ltbPanelBack.Width - Margin
	y0 = 6dip
	setMax(100)
	
	' initialize the background canvas and draw the background line
	cvsPanelBack.Initialize(ltbPanelBack)
	cvsPanelBack.DrawLine(x0, y0, x1, y0, BackLineColor, 2)
	ltbPanelBack.Invalidate
	
	' initialize the foreground canvas
	cvsPanelFront.Initialize(ltbPanelFront)
	
	' set the left cursor parameters and draw it
	PosIndex = 0
	PositionPixels(0) = x0
	DrawPos(x0, 0)	

	' set the right cursor parameters and draw it
	PosIndex = 1
	PositionPixels(1) = x1
	DrawPos(x1, 0)	
End Sub

Private Sub DrawPos(x As Int, Mode As Int)
	' draw a transparent rectangle to erase the foreground panel
	cvsPanelFront.DrawRect(rectPanelFront, Colors.Transparent, True, 1)
	
	' set the current cursor position
	PositionPixels(PosIndex) = x
	
	' define the left cursor path according to its current position
	Paths(0).Initialize(PositionPixels(0), y0)
	Paths(0).LineTo(PositionPixels(0), y0 + 22dip)
	Paths(0).LineTo(PositionPixels(0) - 12dip, y0 + 22dip)
	Paths(0).LineTo(PositionPixels(0) - 12dip, y0 + 8dip)
	Paths(0).LineTo(PositionPixels(0), y0)
	
	' define the right cursor path according to its current position
	Paths(1).Initialize(PositionPixels(1), y0)
	Paths(1).LineTo(PositionPixels(1), y0 + 22dip)
	Paths(1).LineTo(PositionPixels(1) + 12dip, y0 + 22dip)
	Paths(1).LineTo(PositionPixels(1) + 12dip, y0 + 8dip)
	Paths(1).LineTo(PositionPixels(1), y0)
	
	' draw the two cursors and the foreground line
	cvsPanelFront.DrawPath(Paths(0), LimitSliderColor, True, 1)
	cvsPanelFront.DrawPath(Paths(1), LimitSliderColor, True, 1)
	cvsPanelFront.DrawLine(PositionPixels(0), y0, PositionPixels(1), y0, FrontLineColor, 3dip)
	' if Mode = 1 draw the current cursor with the FrontLineColor (highlighted)
	If Mode = 1 Then
		cvsPanelFront.DrawPath(Paths(PosIndex), FrontLineColor, True, 1)
	End If
	ltbPanelFront.Invalidate	' update the foreground panel
End Sub

Private Sub ltbPanelFront_Touch (Action As Int, X As Float, Y As Float)
	' check if the cursor is outsides the limits
	X = Max(x0, X)
	X = Min(x1, X)
	
	' select the Action type
	Select Action
	Case 0  				'DOWN
		If X < Abs(PositionPixels(0) + PositionPixels(1)) / 2 Then
			' if X is closer to the left cursor we choose it
			PosIndex = 0
		Else
			' otherwise we choose the right cursor
			PosIndex = 1
		End If
		DrawPos(X, 1) ' we draw the current cursor highlighted
	Case 2					' MOVE
		DrawPos(X, 1)	' we draw the current cursor highlighted
	Case 1					' UP
		DrawPos(X, 0)	' we draw the current cursor not highlighted
	End Select
	
	' we calculate the current limit value from the X position in pixels
	If PosIndex = 0 Then
		cLimitLeft = Floor((X - x0) / Scale + .5)
	Else
		cLimitRight = Floor((X - x0) / Scale + .5)
	End If
	
	' when Action is UP check if cLimitLeft > cLimitRight 
	' if yes we invert the limit values and redraw the cursors
	If Action = 1 And cLimitLeft > cLimitRight Then
		Private val As Int
		val = cLimitLeft
		cLimitLeft = cLimitRight
		cLimitRight = val
		PosIndex = 0
		X = cLimitLeft * Scale + x0
		DrawPos(X, 0)
		PosIndex = 1
		X = cLimitRight * Scale + x0
		DrawPos(X, 0)
	End If
	
	' if a callback routine exists in the calling module we call it
	If SubExists(Callback, EventName & "_ValuesChanged") Then			'<=== EXCEPTION (Not Using Layout): java.lang.ClassCastException: anywheresoftware.b4a.BALayout cannot be cast to java.lang.String
		CallSub3(Callback, EventName & "_ValuesChanged", cLimitLeft, cLimitRight)
	End If
End Sub

'gets or sets the max value
Public Sub setMax(cMax As Int)
	MaxVal = cMax
	Scale = (x1 - x0) / MaxVal
End Sub

Public Sub getMax As Int
	Return MaxVal
End Sub

'gets or sets the left limit
Public Sub setLimitLeft(Pos As Int)
	' if Pos is lower than 0 set cLimitLeft to 0
	cLimitLeft = Max(0, Pos)
	PosIndex = 0
	DrawPos(x0 + cLimitLeft * Scale, 0)	
End Sub

Public Sub getLimitLeft As Int
	Return cLimitLeft
End Sub

'gets or sets the Left property
Public Sub setLeft(Left As Int)
	ltbPanelBack.Left = Left
End Sub

Public Sub getLeft As Int
	Return ltbPanelBack.Left
End Sub

'gets or sets the Top property
Public Sub setTop(Top As Int)
	ltbPanelBack.Top = Top
End Sub

Public Sub getTop As Int
	Return ltbPanelBack.Top
End Sub

'gets or sets the Width property
Public Sub setWidth(Width As Int)
	ltbPanelBack.Width = Width
	ltbPanelFront.Width = Width
	' initialize the foreground panel rectangle used to erase ltbPanelFront
	rectPanelFront.Initialize(0, 0, ltbPanelFront.Width, ltbPanelFront.Height)	
End Sub

Public Sub getWidth As Int
	Return ltbPanelBack.Width
End Sub

'gets or sets the Height property
Public Sub setHeight(Height As Int)
	ltbPanelBack.Height = Height
	ltbPanelFront.Height = Height
	' initialize the foreground panel rectangle used to erase ltbPanelFront
	rectPanelFront.Initialize(0, 0, ltbPanelFront.Width, ltbPanelFront.Height)
End Sub

Public Sub getHeight As Int
	Return ltbPanelBack.Height
End Sub

'gets or sets the right limit
Public Sub setLimitRight(Pos As Int)
	' if Pos is higher than MaxVal set cLimitRight to MaxVal
	cLimitRight = Min(MaxVal, Pos)
	PosIndex = 1
	DrawPos(x0 + cLimitRight * Scale, 0)	
End Sub

Public Sub getLimitRight As Int
	Return cLimitRight
End Sub

'gets or sets the Visible property
Public Sub setVisible(IsVisible As Boolean)
	ltbPanelBack.Visible = IsVisible
End Sub

Public Sub getVisible As Boolean
	Return ltbPanelBack.Visible
End Sub

